function z = amplmod(f0,x,hub)
% z = amplmod(f0,x,hub)
% Demonstriert die Amlitudenmodulation.
% Ausgegeben wird das modulierte Signal. 
% Wird die Ausgabe einer Variablen zugewiesen, so unterbleibt 
% die grafische Darstellung.
% Es wird ein Zeitabschnitt von t = 0...1 gezeigt.
% f0 ist die Trgerfrequenz. Fr die Demonstration ist f0 = 50 gut.
% x ist ein String, der das Signal als Funktion in MATLAB-Syntax mit 
% der Zeitvariablen t angibt.  Alternativ kann ein Vektor angegeben 
% werden.  Die Modulation ist 100% bei abs(x)=1.
% Gezeigt wird:
% Oberes Bild:     das zu modulierende Signal
% Mittleres Bild:  das modulierte Signal
% Unteres Bild:    das demodulierte Signal
% Die Demodulation erfolgt durch Betragsbildung des modulierten Signals,
% d.h. Wandlung in das analytische Signal, dann sqrt(real^2 + imag^2)
% Beispiel:  amplmod(50,'0.5*sin(2*pi*3*t)+0.3*cos(2*pi*5*t)',1)

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de
%
% Defaults
% ========
if nargin<3 hub = 1; end
if nargin<2 x = '0.5*sin(2*pi*3*t)+0.3*cos(2*pi*5*t)'; end
if nargin<1 f0 = 50; end
ts    = 1.0;                                  % Lnge des Zeitsegments
fsiz  = 14;
hlcol = 0.7*[1 1 1];
patchcol = 0.8*[1 1 1];
if ischar(x) n = 512; else n = length(x); end
g = 30;                                       % Filtergrad fr Hilbertfilter
mm= 3;
m = g/2+mm;                                   % Einschwingbereich
%
% Signalerzeugung
% ===============
t = ts*(0:n+m-1)/n;                           % Zeitsegment
if ischar(x) s = eval(x); else s = [x,zeros(1,m)]; end     % Signal
z = (1 + hub*s).*cos(2*pi*f0*t);              % Modulation
%
if nargout>0 break; end
%
% Demodulation
% ============
y = real2analyt(z,g);                         % Empfangssignal analytisch
a = abs(y)-1;                                 % Demodulation
%
% Wegschneiden des Einschwingbereiches
% ====================================
n = length(t);
t = t(mm+1:n-m+1); s = s(mm+1:n-m+1); z = z(mm+1:n-m+1); a = a(m:n-mm);
%
% Darstellung
% ===========
d  = 0.05;
fh = fullscreen('Amplitudenmodulation');
axes('Position',[0  2/3  1  1/3-d])
plot(t,s); line([0 1],[0 0],'LineStyle',':','Color',hlcol);
axis([0.01  1  1.2*[-1 1]]); axis off
if ischar(x) title(x,'FontSize',fsiz); else title('Signal','FontSize',fsiz); end
axes('Position',[0  1/3  1  1/3-d])
patch([t,reverse(t)],[1+hub*s,-reverse(1+hub*s)],patchcol,'EdgeColor','w'); 
line([0 1],[ 1  1],'LineStyle',':','Color',hlcol);
line([0 1],[-1 -1],'LineStyle',':','Color',hlcol);
line(t,z)
set(gca,'XLim',[0.01,1]); axis off
title('Amplitudenmodulation','FontSize',fsiz)
axes('Position',[0  0  1  1/3-d])
plot(t,a); line([0 1],[0 0],'LineStyle',':','Color',hlcol);
axis([0.01  1  1.2*[-1 1]]); axis off
title('demoduliertes Signal','FontSize',fsiz)
