function a = filterplot(b,f1,f2,vs,d1,d2,dez,p,ser,fs)
% a = filterplot(b,f1,f2,vs,d1,d2,dez,p,ser,fs)
% plottet den Betragsfrequenzgang von mehreren FIR- und IIR-Filtern, 
% die mit unterschiedlichen Raten berechnet werden knnen.
%
% Parameter:
%  b:     Cell Array aller Koeffizientenvektoren:  {b1; b2; b3; ...}
%         wo  b1 = Koeffizienten von Filter1; b2 = Koeff. ...
%         Im Falle nur eines FIR-Filters kann auch der Koeffizienten-
%         Vektor direkt, also ohne {...} angegeben werden.
%         Anstelle eines Koeffizientenvektors muss im Falle eines
%         IIR-Filters das Cell Array  {bi; ai} treten.
%  f1,f2: untere und obere Frequenz fr den Plot (0 <= f1 < f2 <= 1).
%         Diese Frequenzen sind relativ zur hchsten benutzten
%         Taktfrequenz/2, z.B.  f1 = 0;  f2 = 1 fr den gesamten Bereich
%  vs:    wenn  vs = 'lin' , dann Skalierung linear, sonst in dB (z.B.vs=0)
%  d1,d2: vertikale Plotgrenzen in dB (d1 < d2), z.B. -120,+10
%  dez:   Zeilenvektor, der die Dezimationszahlen der Filter enthlt;
%         z.B.  [1, 2, 4].  Default ist [1  1  1 ...]
%         Wird ein Skalar dez angegeben, so gilt dieser fr alle Filter.
%  p:     Begrenzung der Reprsentation der Koeffizienten auf p Bits.
%         Dabei wird eine mgliche Skalierung, wie sie von KOFSAVE 
%         vorgenommen wird, bercksichtigt. Default ist p = 24.
%         Ist p eine Zahl, so wird dies fr alle Filter angenommen.
%         Ist p ein Vektor, so gelten die Komponenten fr die 
%         enzelnen Filter der Reihe nach.
%  ser:   Ist ser = 1 , dann wird der Frequenzgang der Serienschaltung
%         aller in b enthaltenen Filter gezeichnet.
%  fs :   Abtastfrequenz in Hz.  Falls fs angegeben, so erfolgt die 
%         Frequenzangabe in Hz anstelle von Omega.
%
% Falls eine Zuweisung auf a erfolgt, so gibt es keine Graphik.
%  a:     Matrix, die die zu plottenden Kurven diskret als Zeilen enthlt.
%         Zeile 1 enthlt die zugehrigen Frequenzen.
%         Alle Werte gehorchen den Parametern, insbesondere vs uns ser !
%
% Beispiele:
% b=remez(30,[0,0.3,0.7,1],[1,1,0,0],[0.001,1]);
% [b2,a2]=ellip(5,1,60,0.3)
% filterplot({b;b;b;b;b},0,1,0,-150,10,[1 1 1 1 1],[3,6,12,24,100])
% filterplot({b;b;b},0,1,0,-150,10,[1 2 4])
% filterplot({b;b;b},0,1,0,-200,10,[1 2 4],24,1)
% filterplot({b},0,0.33,0,-0.002,0.002)
% filterplot({b;{b2;a2}},0,1,'lin',0,1.1)
% a = filterplot({b;{b2;a2}},0,1,'lin',0,1.1); plot(a(1,:),a(2:end,:))

% K. von der Heide,  e-mail:  v.d.heide@on-line.de
%
% Grssen
% =======
n = 800;                  % Anzahl der Frequenzpunkte fr die Darstellung
if nargin<1    b = {remez(30,[0,0.3,0.7,1],[1,1,0,0],[0.001,1])}; end
if ~iscell(b)  b = {b}; end
[bz,bs] = size(b);        % bz = Anzahl der darzustellenden Filter
fpt = 12;                 % FontSize
%
% Defaults
% ========
if nargin<9  ser = 0; end
if nargin<8  p = 24;  end
if nargin<7  dez = ones(1,bz); end
if nargin<4  vs = 'db'; end
if nargin<6  d2 = 10;   if strcmp(vs,'lin')  d2 = 1; end; end
if nargin<5  d1 = -100; if strcmp(vs,'lin')  d1 = 0; end; end
if nargin<3  f2 = 1;  end
if nargin<2  f1 = 0;  end
if length(p)==1  pp = p*ones(1,bz); else  pp = p; end
if length(dez)==1  dz = dez*ones(1,bz); else dz = dez; end
%
% Darstellung
% ===========
ff = 1/n*((1:n)-0.5);
ff = (f2-f1)*ff+f1;       % Frequenzbereich fr die Darstellung
zz = ones(size(ff));
for k = 1:bz              % Frequenzgang berechnen fr alle Filter
   if iscell(b{k})  bb = b{k}{1}; aa = b{k}{2};
   else             bb = b{k};    aa = 1;  end
   yy = max(1E-20,abs(freqz(limprec(bb,pp(k)+min([7,...
            floor(-log10(max(bb))/log10(2))])),limprec(aa,pp(k)+min([7,...
            floor(-log10(max(aa))/log10(2))])),ff*dz(k)*pi)));
   zz = zz.*yy;
   if ~strcmp(vs,'lin') y(k,:) = 20*log10(yy); else y(k,:) = yy; end
   for kk = 1:n  if isnan(y(k,kk))  y(k,kk) = 1E-20; end; end
end

if nargout>0              % Zuweisung der Werte anstelle der Graphik
   if ser~=1  a = [ff;y];
   else if ~strcmp(vs,'lin')  a = [ff;20*log10(zz)]; else a = [ff;zz]; end;
   end
   break      
end

if ser~=1  plot(ff,y)
else if ~strcmp(vs,'lin') plot(ff,20*log10(zz)); else plot(ff,zz); end; 
end
axis([f1,f2,d1,d2]);
grid('on');
yl = 'Betragsfrequenzgang';
if ~strcmp(vs,'lin')  yl = strcat(yl,' (dB)'); end
ylabel(yl,'FontSize',fpt);
if nargin<10
   xlabel('\Omega','FontSize',fpt);
else
   xlabel('Frequenz [Hz]','FontSize',fpt);
   set(gca,'XTickLabel',deblank(num2str(0.5*fs*str2num(get(gca,'XTickLabel')))))
end
