function phasedarray(xy,t,f,c,n,m)
% phasedarray(xy,t,f,c,n,m)
% Darstellung der Richtwirkung eines Phased Arrays von Antennen oder 
% Mikrophonen an den x-y-Positionen xy fr Frequenzen im Intervall f(1)...f(2).
%
% Parameter (Defaults in Klammern):
% xy  :  2*anz-Matrix [anz = Anzahl der Mikrophone bzw. Antennen]
%        xy(1,:)  x-Positionen in m (16 zufllige Werte -0.08 ... +0.08)
%        xy(2,:)  y-Positionen in m (16 zufllige Werte -0.08 ... +0.08)
% t   :  Vektor der Lnge anz mit den Zeitverschiebungen fr die 
%        Speisung der Antennen in s (xy(1,:)/c fr Keule in x-Richtung)
% f   :  Vektor mit den zwei Eckfrequenzen in Hz ([0 1500])
% c   :  Phasengeschwindigkeit in m/s  (340)
% n   :  Anzahl der Teilpunkte des Vollkreises (90)
% m   :  Anzahl der Teilpunkte des Frequenzintervalls (20)
%
% Der Aufruf ohne Parameter liefert das typische Stiefel-Diagramm fr 
% eine zufllige Anordnung von 128 Mikrophonen in einem Quadrat der
% Kantenlnge 1 m.

% Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<6   m = 20;  end
if nargin<5   n = 90;  end
if nargin<4   c = 340; end
if nargin<3  fe = [0 1500]; end
if nargin<1  xy = 0.5*(rand(2,128)-0.5); end
[dummy,anz] = size(xy);
if nargin<2   t = xy(1,:)/c; end

% Richtungsraster und Frequenzraster
% ==================================
a = (0:n)/n*2*pi;
f = (0:m)/m*(fe(2)-fe(1)) + fe(1);

% Berechnung der Richtwirkung
% ===========================
tc = c*t;
for j=1:n+1
   D  = [cos(a(j)) -sin(a(j)); sin(a(j)) cos(a(j))];
   d  = D*xy;
   dx = d(1,:);
   for k=1:m+1
      g(j,k) = abs(sum(cos(2*pi*(dx-tc)*f(k)/c)));
   end
end

% Umrechnung in Zylinderkoordinaten
% =================================
for k=1:m+1  
   [x(k,:),y(k,:)] = pol2cart(a,g(:,k)'); 
   z(k,:) = f(k)*ones(1,n+1);
end

% Graphik
% =======
%fh = fullscreen('PhasedArray');
fh = figure;
for k=1:m
   for j=1:n
      xx = [x(k,j:j+1) x(k+1,j+1:-1:j)];
      yy = [y(k,j:j+1) y(k+1,j+1:-1:j)];
      rr = sqrt(xx.^2 + yy.^2)/anz;
      patch(xx,yy,[z(k,j:j+1) z(k+1,j+1:-1:j)],rr*64)
   end
end
axis square
title('Richtwirkung eines Mikrophon-Arrays in Abhngigkeit der Frequenz','FontSize',13)
zlabel('Frequenz [Hz]','FontSize',13)
set(gca,'View',12*[-1 +1])
rotate3d
