function x0 = plot_code_gain(code)
% x0 = plot_code_gain(code)
% Wahrscheinlichkeit fr Bitfehler der dekodierten Bits unter
% Bercksichtigung, dass zur bertragung eines Informationsbits nur die
% Energie Eb zur Verfgung steht und dass die strende Rauschenergie 
% proportional zu n ist. Das Rauschen sei Gauss-verteilt.
%
% code ist eine Matrix, deren Zeilen fr die zu zeichnenden Kodes 
% jeweils die Angaben n,k,r enthlt, wo 
% n = Kodewortlnge
% k = Anzahl Informationsbits
% r = maximale Zahl zu korrigierender Bits.  
%
% Beispiel: plot_code_gain([1,1,0;7,4,1;3,1,1;23,12,3;32,6,7]); zeichnet die 
% Fehlerwahrscheinlichkeit fr n=k=1, r=0 also unkodierte bertragung 
% fr (7,4)-Hammingkode und (3,1)-Wiederholungskode  mit 1-Bit-Korrektur,
% fr den (23,12)-Golay-Kode mit 3-Bit-Korrektur 
% sowie fr den (32,6)-Reed-Muller-Kode mit 7-Bit-Korrektur (Mariner-Sonden).
%
% Anmerkung: Neben der hier benutzten Methode der Korrektur von falsch 
% empfangenen Bits gibt es die Korrelationsmethode, die das Kodewort mit
% dem geringsten euklidischen Abstand sucht. Diese erfordert eine 
% Aufzeichnung der den Bits entsprechenden analogen Werte. Man gewinnt 
% damit unabhngig vom Kode pi/2 bzw. 1.96 dB.

% K. von der Heide, 18.11.1998,  e-mail: v.d.heide@on-line.de

% Konstanten
% ==========
fsiz = 13;                 % FontSize
lw   = 1.5;                % LineWidth
if nargin<1 code = [1,1,0;7,4,1;3,1,1;23,12,3;32,6,7]; end

% Berechnung
% ==========
EbNodB = (0:720)/40 - 5;   % Abszisse fr das Diagramm: Verhltnis Eb/No in dB
[cc,i] = size(code);       % Grsse der Eingabematrix
for c = 1:cc               % Berechnung fr alle gewnschten Kodes
   n = code(c,1); k = code(c,2); r = code(c,3);
   p = 0.5*(1-erf(sqrt(k/n*(10.^(EbNodB/10))))); % Umrechnung Eb/No => Bitfehlerwahrsch.
   if r == 0  ph(c,:) = p; % bei unkodierten Bits p direkt nehmen
      else for i = 1:length(EbNodB)  ph(c,i) = (1 - p_dec_ok(n,r,p(i)))/2; end;
   end;                    % Faktor 1/2 weil das Ergebnis bei Decodierfehler zufllig
   if c == 1  legende = sprintf('(%g,%g)-Kode mit %g-Bit-Korrektur',n,k,r);
      else    legende = char(legende, sprintf('(%g,%g)-Kode mit %g-Bit-Korrektur',n,k,r));
   end;
end;

% Plot
% ====
fh  = fullscreen('Bitfehlerwahrscheinlichkeit');
hlo = plot(EbNodB,ph);
set(hlo,'LineWidth',lw)
axis([-5,13,max(10^-12,min(min((ph)))),1]);
xlabel('E_b/N_0 in dB','FontSize',fsiz);
ylabel('Bitfehlerwahrscheinlichkeit fr dekodierte Bits','FontSize',fsiz);
grid on;
set(gca,'YScale','log','FontSize',fsiz);
legend(legende,3);
