function [x0,y0,z0] = schrittsync(txschritt,rxschritt,txsymb,sigma,neutral,tol)
% [x0,y0,z0] = schrittsync(txschritt,rxschritt,txsymb,noise,neutral)
% Schrittsynchronisation nach Maximum Likelihood:
% Zunchst wird eine erste Schtzung der Schrittfrequenz aus dem Spektrum 
% des Signalquadrates gewonnen. Anschliessend erfolgt eine Ausgleichung 
% nach der Methode der kleinsten Quadrate. Die Fehlergleichungen werden 
% linearisiert und der Gauss'sche Algorithmus nach der Newton'schen Methode 
% iteriert.
%
% Parameter:
% txschritt     :  Symbolschritt in Samples (sinvoll 4...100)
% rxschritt     :  Schtzung fr den Empfnger-Symbolschritt
% txsymb        :  Vektor der zu sendenden Symbole
% sigma         :  Standardabweichung des weissen Kanalrauschens
% neutral       :  empfangenes Symbol, wenn keine Sendung
% tol           :  Toleranz fr die Scrittfrequenz (1-tol)...(1+tol)
% x0            :  ermittelter Empfnger-Symbolschritt
% y0            :  emfangene Bits {-1 +1} bei Entscheidung durch >0 (ohne Rauschen)
% z0            :  emfangene Bits {-1 +1} bei Entscheidung durch >0 (mit Rauschen)
% Defaults sind :  txschritt = 100;  rxschritt = 110; txsymb = 16 zufllige Bits;
%                  sigma = 0.1; neutral = 1; tol = 0.1;

% Autor: K.von der Heide,  e-mail: v.d.heide@on-line.de

% Defaults
% ========
if nargin<6       tol = 0.1; end
if nargin<5   neutral = 1;   end
if nargin<4    sigma  = 0.1; end
if nargin<3   txsymb  = 2*randb(16)-1; end
if nargin<2 rxschritt = 110; end
if nargin<1 txschritt = 100; end
anzsymb = length(txsymb);
symbole = [neutral, txsymb, neutral];  % zu sendende Symbole
anziter = 3;                           % Anzahl der Iterationen
relax   = 1;                           % Relaxationsfaktor
fsiz    = 12;                          % FontSize
b = fir1(100,1.3/rxschritt);   

% Sender
% ======
vorlauf= neutral*ones(1,ceil(txschritt*rand)); % zuflliger Zeitversatz
auslauf= neutral*ones(1,txschritt-length(vorlauf)); % Auslauf
txsig  = [vorlauf,raisedcosine(symbole,txschritt),auslauf];

% Kanal
% =====
rxsig  = txsig + sigma*randn(1,length(txsig)); % Kanalrauschen
       
% Empfnger
% =========
sigsqr = rxsig.^2;                     % Signal quadrieren
sigsqw = sigsqr - mean(sigsqr);        % Mittelwert subtrahieren
siglng = length(rxsig);                % Lnge der Sequenz
zeitbr = 0:siglng - 1;                 % Zeitbereich der Sequenz

% Schrittsynchronisation
% ======================
% erste Nherung der Schrittfrequenz durch Maximum im Spektrum
fftsqw = abs(fft(sigsqw));             % FFT
tolber = 1 + (round((1-tol)*siglng/rxschritt):round((1+tol)*siglng/rxschritt));
schrfr = (find(fftsqw(tolber)==max(fftsqw(tolber)))+tolber(1)-2)/siglng;
costab = cos(2*pi*schrfr*zeitbr);      % cos-Tabelle
sintab = sin(2*pi*schrfr*zeitbr);      % sin-Tabelle
% Methode der kleinsten Quadrate (Anfangswerte fr cosamp, sinamp):
amat   = [costab; sintab]';
csamp  = inv(amat'*amat)*amat'*sigsqw';% Gauss-Algorithmus
cosamp = csamp(1);  
sinamp = csamp(2);
phi    = angle(cosamp + i*sinamp);     % Phasenwinkel
if phi<0  phi = phi + 2*pi; end        % 0 < phi < 2*pi
abtast = round(1+rxschritt*phi/(2*pi):rxschritt:siglng); % Abtastpunkte
if abtast(1)==0  abtast = abtast(2:length(abtast)); end
gew= [zeros(1,abtast(1)),ones(1,abtast(length(abtast))-abtast(1)-1),...
      zeros(1,siglng-abtast(length(abtast))+1)];
% Methode der kleinsten Quadrate (Newton-Iteration):
for k=1:anziter
   amat   = [costab; sintab; 2*pi*(sinamp*costab - cosamp*sintab).*zeitbr.*gew]';
   rseite = sigsqw - cosamp*costab - sinamp*sintab;
   verbes = relax*inv(amat'*amat)*amat'*rseite'; % Gauss-Algorithmus
   cosamp = cosamp + verbes(1);        % Verbesserungen anbringen
   sinamp = sinamp + verbes(2);
   schrfr = schrfr + verbes(3);
   rxschr = 1/schrfr;                  % verbesserter Empfngerschritt
   costab = cos(2*pi*schrfr*zeitbr);   % verbesserte cos-Tabelle
   sintab = sin(2*pi*schrfr*zeitbr);   % verbesserte sin-Tabelle
   phi    = angle(cosamp + i*sinamp);  % Phasenwinkel
   if phi<0  phi = phi + 2*pi; end     % 0 < phi < 2*pi
   abtast = round(1+rxschr*phi/(2*pi):rxschr:siglng); % Abtastpunkte
   if abtast(1)==0  abtast = abtast(2:length(abtast)); end
   gew= [zeros(1,abtast(1)),ones(1,abtast(length(abtast))-abtast(1)-1),...
         zeros(1,siglng-abtast(length(abtast))+1)];
end
vdiff = abs(abtast-length(vorlauf));
tast1 = find(vdiff==min(vdiff))+1;
x0 = rxschr;
tst= tast1:length(txsymb)+tast1-1;
if tst(length(tst))>length(abtast) 
   y0 = 0;
   z0 = 0;
else
   y0 = 2*(txsig(abtast(tst))>0)-1;
   rb = conv(b,rxsig);
%   z0 = 2*(rb(abtast(tst+length(rb)-length(rxsig)))>0)-1;
   z0 = 2*(rb(abtast(tst)+50)>0)-1;
end

% Darstellung
% ===========
% Spektrum des quadrierten Signals:
h1 = fullscreen('Spektrum','w',2,[2 1;2 1]);
set(h1,'keypressfcn','zoom3d')
sp = spctrm(sigsqw);
semilogx((0:length(sp)-1)/length(sp)/2,sp)
set(gca,'xlim',[0.001,0.5])
title('Spektrum des quadrierten Empfangssignals','FontSize',fsiz)
xlabel('Frequenz / Abtastfrequenz','FontSize',fsiz)
% Zeitsignale
mx = max([abs(min(rxsig)),abs(max(rxsig))]) + 0.5;
sigtast = round(length(vorlauf)+1:txschritt:siglng);

h2 = fullscreen('Schrittsync','w',2,[2 1; 1 1]);
set(h2,'KeyPressFcn','xzoomvis')
axis([0, siglng-1, -mx, +mx])
% korrekte Abtastpunkte (cyan)
line(sigtast-1,txsig(sigtast),'LineStyle','none','Marker','o',...
     'MarkerEdgeColor','c','MarkerSize',8,'LineWidth',2)
% Null-Linie (grau gestrichelt)  
line([0 siglng],[0 0],'LineStyle',':','Color',0.7*[1 1 1])
% Sendesignal (cyan)
line(zeitbr,txsig,'Color','c')
% Empfangssignal (blau)
line(zeitbr,rxsig,'Color','b')
% Signalquadrat (grn)
line(zeitbr,sigsqr*(mx-0.5)/max(sigsqr),'Color','g')
% rekonstruierter Schritt-Sinus (rot)
amp = sqrt(cosamp^2 + sinamp^2);
line(zeitbr,gew.*((cosamp*costab + sinamp*sintab)/amp + 1)/2,'Color','r')
% rekonstruierte Abtastpunkte (rote Sterne)
line(abtast-1,txsig(abtast),'LineStyle','none','Marker','*',...
   'MarkerEdgeColor','r')
% Legende
hl = legend('1 korrekte Abtastwerte','2 Null-Linie','3 Sendesignal',...
   '4 Empfangssignal','5 Quadrat des Empfangs.',...
   '6 rekonstr. Schritt-Takt','7 aktuelle Abtastpunkte');
set(hl,'Position',[0.0 -0.0028 0.2641 0.33])
xlabel('die Sichtbarkeit der einzelnen Linien kann durch die Tasten 1...7 getoggelt werden',...
       'FontSize',fsiz)

