function z = ssbmod(f0,x,bnd,f1,ph)
% ssbmod(f0,x,bnd,f1,ph)
% Demonstriert die Einseitenbandmodulation ohne Trger.
% Ausgegeben wird das modulierte Signal. 
% Wird die Ausgabe einer Variablen zugewiesen, so unterbleibt 
% die grafische Darstellung.
% Es wird ein Zeitabschnitt von t = 0...1 gezeigt.
% f0 ist die Trgerfrequenz. Fr die Demonstration ist f0 = 50 gut.
% x ist ein String, der das Signal als Funktion in MATLAB-Syntax mit 
% der Zeitvariablen t angibt. Alternativ kann ein Vektor angegeben 
% werden.
% bnd = 1 :   oberes Seitenband
% bnd =-1 :  unteres Seitenband
% Gezeigt wird:
% Oberes Bild:     das zu modulierende Signal
% Mittleres Bild:  das modulierte Signal
% Unteres Bild:    das demodulierte Signal
% Zum demodulieren wird die Trgerfrequenz f1 und die Phase ph verwendet 
% (Defaults:  f1 = f0, ph = pi).
% bnd<0 : unteres Seitenband (LSB);  bnd>0 : oberes Seitenband (USB)
% Beispiel:  ssbmod(50,'0.5*sin(2*pi*3*t)+0.3*cos(2*pi*5*t)',1)

% K. v. d. Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<3 bnd= 1;  end
if nargin<2  x = '0.5*sin(2*pi*3*t)+0.3*cos(2*pi*5*t)'; end
if nargin<1 f0 = 50; end
if nargin<4 f1 = f0; end                      % Default fr Demodulatortrger
if nargin<5 ph = pi-pi*f0/17.3; end

ts    = 1.0;                                  % Lnge des Zeitsegments
hlcol = 0.7*[1 1 1];                          % Hilfslinienfarbe grau
patchcol = 0.8*[1 1 1];
fsiz  = 14;                                   % FontSize fr Titel
if ischar(x) n = 512; else n = length(x); end
g = 80;                                       % Filtergrad fr Hilbertfilter

% Signalerzeugung
% ===============
t = ts*(0:n+g-1)/n;                           % Zeitsegment
if ischar(x) s = eval(x); else s = [x,zeros(1,g)]; end     % Signal
sa= real2analyt(s,g,0.024);                   % Input als analytisches Signal
if bnd>0 z = real(sa.*exp(i*2*pi*f0*t));      % Modulation USB
else     z = real(sa.*exp(-i*2*pi*f0*t));     % Modulation LSB
end 

if nargout>0 break; end

% Demodulation
% ============
zz = real2analyt(z,g);
w  = real(zz.*(cos(2*pi*f1*t+ph) - i*sin(2*pi*f1*t+ph)));

% Wegschneiden des Einschwingbereiches
% ====================================
n = length(t);
m1= floor(g/2)+1;
m2= g/2;
t = t(1:n-m1-m2+1); s = s(1:n-m1-m2+1); sa = sa(m1:n-m2); 
z = z(m1:n-m2);     w = w(m1+m2:n);

if nargout>0  break; end

% Darstellung
% ===========
if bnd>0 fname = 'SSB (USB)'; else fname = 'SSB (LSB)'; end
d  = 0.05;
fh = fullscreen(fname);
axes('Position',[0  2/3  1  1/3-d])
plot(t,s); line([0 1],[0 0],'LineStyle',':','Color',hlcol);
set(gca,'XLim',[0.01,0.99]); axis off
if ischar(x) title(x,'FontSize',fsiz); else title('Signal','FontSize',fsiz); end
axes('Position',[0  1/3  1  1/3-d])
patch([t,reverse(t)],[abs(sa),-reverse(abs(sa))],patchcol,'EdgeColor','w'); 
line(t,z)
set(gca,'XLim',[0.01,0.99]); axis off
title(fname,'FontSize',fsiz);
axes('Position',[0  0  1  1/3-d])
plot(t,w); line([0 1],[0 0],'LineStyle',':','Color',hlcol);
set(gca,'XLim',[0.01,0.99]); axis off
title('demoduliertes Signal','FontSize',fsiz)
