function  [x0,y0] = txrxfilter(bs,be,EbN0,fs,n,fig,pr,pa)
% [x0,y0] = txrxfilter(bs,be,EbN0,fs,n,fig,pr,pa)
% Simulation digitaler bertragung im Basisband mit 
% weissem Rauschen und zufllig verteilten Impulsstrungen.
%
% Ausgabe:
% x0   die Anzahl in der Auswertung bertragener Bits
% y0   die Anzahl fehlerhafter Bits unter den x0 Bits
%
% Parameter: 
% bs   Koeffizienten des FIR-Filters zur Sendepulserzeugung
% be   Koeffizienten des Empfangs-FIR-Filters
% EbN0 Eb/N0 Signal/Rauschverhltnis 
% fs   Anzahl der Abtastungen in einem Symbolschritt
% n    Anzahl zu bertragender Bits (Zufallsmuster)
% fig  Bilder werden nur gezeigt, wenn fig = 1
% pr   gibt die Rate von Impulsstrungen relativ zu fs an,
% pa   ist die Standardabweichung der gaussverteilten 
%      Impulsamplituden.
%
% Die Faltung von Sende- und Empfangsfilter sollte 
% die erste Nyquistbedingung erfllen, also keine 
% Intersymbolinterferenzen aufweisen. Deshalb wird 
% das Gesamtfilter im Bild gezeigt.

% Autor: Klaus von der Heide; e-mail: v.d.heide@on-line.de

% Defaults und Konstanten
% =======================
if nargin<1  bs  = [ones(1,10)]/10;  end
if nargin<2  be  = crfir(12,0.11,1); end
if nargin<3  EbN0= 20000; end
if nargin<4  fs  = 10; end
if nargin<5  n   = 53; end
if nargin<6  fig = 1;  end
if nargin<7  pr  = 0;  end
if nargin<8  pa  = 0;  end
gs = length(bs) - 1;
ge = length(be) - 1;
ns = n*fs;
nsr= ns - ceil(ge/2);
an = 2*fs + 1;
t  = an:nsr;

% Generierung eines Digitalsignals im Schrittakt
% ==============================================
d = 2*randb(n) - 1;

% Abtastung des Rechteck-Digitalsignals mit fs
% ============================================
ds = [zeros(1,ceil(fs/2)),nullein(d,fs-1),zeros(1,floor(fs/2)-1)];

% Sendefilter
% ===========
x  = filter(bs,1,ds)/max(abs(bs)); % Sendesignal

% Kanalsimulation durch Addition von weissem Rauschen
% ===================================================
rate  = 1/fs;   % uncodiert, aber fs Samples
sigma = 1/sqrt(2*rate*(10^(EbN0/10)));
y = x + sigma*randn(1,length(x));  % Empfangssignal

% zustzliche Impulsstrungen
% ===========================
for k=1:length(y) if rand<pr y(k) = y(k) + pa*randn; end; end

% Empfangsfilter
% ==============
z  = filter(be,1,y);               % Signal nach Empfangsfilter

% Datenentscheidung
% =================
delay= round((fs+gs+ge)/2);
daus = 2*(z(delay+1:fs:length(z))>0) - 1;

% Vergleich
% =========
delay= round((gs+ge)/2/fs);
dein = d(1:length(daus));
x0   = length(daus);
y0   = sum(dein-daus~=0);

% Voreinstellung fr Bilder
% =========================
if fig == 1
   switch screen
      case 1
         pos{1} = [1 208 318 371];
         pos{2} = [293 398 239 182];
         pos{3} = [293 208 238 201];
         pos{4} = [4 31 794 186];
         pos{5} = [513 208 285 371];
         pos{6} = [513 208 285 371];
      case 2
         pos{1} = [2 310 315 436];
         pos{2} = [292 542 360 205];
         pos{3} = [291 310 361 207];
         pos{4} = [2 30 652 298];
         pos{5} = [621 383 401 363];
         pos{6} = [621 30 401 349];
      otherwise error('Bildpositionen fr screen nicht angegeben.');
   end;
   for k = 1:6
      h(k) = figure;
      set(h(k),'position',pos{k},'Color','w','MenuBar','none',...
               'NumberTitle','off','keypressfcn','togglevisibility');
   end;
end

% Darstellung
% ===========
if fig == 1
   figure(h(1))                    % Frequenzgang der Filter
   set(h(1),'Name','Frequenzgang der Filter')
   filterplot({bs;be},0,1,0,-100,10)
   legend('Sendefilter','Empfangsfilter')

   figure(h(2))                    % Pulsform 
   set(h(2),'Name','Sendepuls bzw. Impulsantwort')
   bsf = bs;
   bef = be;
   while length(bsf)<length(bef) bsf = [bsf,0]; end
   while length(bef)<length(bsf) bef = [0,bef]; end
   if rem(length(bsf),2)==0
      bsf = [bsf,0];
      bef = [bef,0];
   end
   gr = (length(bsf)-1)/2;
   
   plot((-1-gr:1+gr)/fs,[0,bsf,0],(-1-gr:1+gr)/fs,[0,bef,0])
   for k = 1:floor(gs/2/fs)
      line([-k,-k],[0,0],'LineStyle','none','Marker','.',...
           'MarkerEdgeColor','r','MarkerSize',20);
      line([k,k],[0,0],'LineStyle','none','Marker','.',...
           'MarkerEdgeColor','r','MarkerSize',20);
   end
   set(gca,'YLim',1.2*get(gca,'YLim'))  
   legend('Sendepuls','Empfangsfilter')
   title('Impulsantwort')
   grid on

   figure(h(3))                    % Impuls nach Empfangsfilter
   set(h(3),'Name','Impuls nach dem Empfangsfilter')
   plot((-1-(gs+ge)/2:1+(gs+ge)/2)/fs,[0,conv(bs,be),0])
   grid on
   for k = 1:floor((gs+ge)/2/fs)
      line(-k*[1,1],[0,0],'LineStyle','none','Marker','.',...
           'MarkerEdgeColor','r','MarkerSize',20);
      line(k*[1,1],[0,0],'LineStyle','none','Marker','.',...
           'MarkerEdgeColor','r','MarkerSize',20);
   end
   set(gca,'YLim',1.2*get(gca,'YLim'))  
   title('Gesamtfilter = Faltung von Sende- und Empfangsfilter')

   figure(h(4))                    % Signale im Zeitbereich
   set(h(4),'Name','Signale im Zeitbereich')
   plot(t,x(an:nsr),t,y(an:nsr))
   line(t,z(an+ns-nsr:ns),'color','r','LineWidth',1.5)
   line(t,zeros(1,nsr-an+1),'color',0.7*[1 1 1])
   axis([an,nsr,-3,3])
   legend('gesendetes Signal','empfangenes Signal','nach Empfangsfilter')

   figure(h(5))                    % Augendiagramm
   set(h(5),'Name','Augendiagramm','WindowButtonDownFcn','msvfigbdf','WindowButtonUpFcn','msvfigbuf')
   k  = 3;
   gg = ceil((gs+ge)/2);
   mitte = ceil(fs/2)+1+(gs+ge)/2-gg;
   ta = ((-1:fs+3) - mitte)/fs;
   while (gs+ge)/2+(k+1)*fs+2 < ns
      line(ta,z(gg+k*fs-1:gg+(k+1)*fs+3));
      k = k + 1;
   end
   line([0 0],[-1.5,1.5],'Color','r');
   line(([-1,fs+3]-mitte)/fs,[0,0],'Color','r');
   drawnow
   axis([[-1-mitte,fs+3-mitte]/fs,-1.5,1.5])
   
   figure(h(6))                    % Spektrum
   set(h(6),'Name','Sendespektrum','KeyPressFcn','hervorhebung','WindowButtonDownFcn','msvfigbdf','WindowButtonUpFcn','msvfigbuf')
   fftx = fft(x.*hamming(length(x))');
   ffty = fft(y.*hamming(length(y))');
   sp   = [abs(fftx(1:round(length(fftx)/2)));abs(ffty(1:round(length(ffty)/2)))];
   plot(fs/2*(0:length(sp)-1)/length(sp),sp)
   title('Spektren')
   xlabel('Frequenz relativ zur Schrittfrequenz')
   legend('gesendetes Signal','empfangenes Signal')
   
   figure(h(1)); figure(h(3)); figure(h(2)); figure(h(5)); figure(h(6))
end
