function x0 = v22bis(snr,n)
% v22bis(snr,n)
% Signalraumdarstellung von CCITT V.22bis im Rauschen von snr [dB].
% Das Signal-Rauschverhltnis snr [dB] bezieht sich auf die mittlere 
% Signalleistung bei gleicher Hufigkeit aller Symbole.
% Vollduplex:  QAM auf f0 = 1200 Hz bzw. 2400 Hz mit 600 Baud.
% Codierung:   Das erste DiBit codiert den Wechsel des Quadranten,
%              das zweite den Punkt im Quadranten.
%              DiBit        Quadrantenwechsel
%               0 0   Quadrant(n+1) = rem(Quadrant(n)+1,3)   
%               0 1   Quadrant(n+1) = Quadrant(n)   
%               1 0   Quadrant(n+1) = rem(Quadrant(n)+2,3)   
%               1 1   Quadrant(n+1) = rem(Quadrant(n)+3,3)   
% Die Darstellung unterbleibt, wenn die Ausgabe einer Variablen 
% zugewiesen wird.  Die Ausgabe x0 enthlt n komplexe Symbole fr
% den Fall 16QAM ohne Rauschen.  Die mittlere Leistung eines 
% Symbols ist 1.0, also Eb = 1/4.

% Autor: K. von der Heide,  e-mail:  v.d.heide@on-line.de

% Konstanten und Defaults
% =======================
fsiz = 15;                                    % FontSize
if nargin<2    n = 15000; end
if nargin<1  snr = 20;    end

% Symbole in CCITT V.22bis
% ========================
a = sqrt(4*16/160);                           % Gitterkonstante fr Eb = 1
sig = [];
for k = 1:4
   sig = [sig, a*(k - 2.5 + i*(-1.5:1:+1.5))];
end

% Signal
% ======
sa = sig(ceil(length(sig)*rand(1,n)));        % Signal

x0 = sa;
if nargout  break; end

% Rauschen
% ========
ra  = (add_awgn(zeros(1,n),1,snr)...           % reell
       + i*add_awgn(zeros(1,n),1,snr))/2;      % imaginr

% Signal + Rauschen
% =================
sr = sa + ra;
mx = max([real(sr),imag(sr)]);

% Darstellung
% ===========
h1 = fullscreen('V.22bis');
subplot(1,2,1)
line(real(sr'),imag(sr'),'LineStyle','none','Marker','.')
line([-mx,0;mx,0],[0,-mx;0,mx],'LineStyle','-','Color','r')
axis([-mx,mx,-mx,mx]); axis square; axis off
title(sprintf('V.22bis 2k4  SNR = %4.1f dB',snr),'FontSize',fsiz)
   
% Symbole in CCITT V.22bis 1k2
% ============================
nn = round(n/4);                     
sig = exp(i*pi*(0:90:359)/180)';

% Signal
% ======
sa = sig(ceil(length(sig)*rand(1,nn)))';      % Signal

% Rauschen
% ========
ra  = add_awgn(zeros(1,nn),1,snr)...          % reell
      + i*add_awgn(zeros(1,nn),1,snr);        % imaginr

% Signal + Rauschen
% =================
sr = sa + ra;
mx = max([real(sr),imag(sr)]);

% Darstellung
% ===========
subplot(1,2,2)
line(real(sr'),imag(sr'),'LineStyle','none','Marker','.')
line([-mx,0;mx,0],[0,-mx;0,mx],'LineStyle','-','Color','r')
axis([-mx,mx,-mx,mx]); axis square; axis off
title(sprintf('V.22bis 1k2  SNR = %4.1f dB',snr),'FontSize',fsiz)
