function d = viterbi(g,x,s,diag)
% d = viterbi(g,x,s,diag)
% Viterbi-Dekodierung von binren Faltungskodes.
% Aufruf ohne Parameter liefert eine interaktive Graphik-Demo.
%
%   g  :   Generatormatrix mit r Faltungspolynomen als Zeilen (Nullen und Einsen)
%
%   x  :   Vektor oder Matrix des zu dekodierenden Signals (reell +-),
%          Ist x ein Zeilenvektor, so wird angenommen, dass die Faltung 
%          mit allen Polynomen verzahnt vorliegt.
%          x kann auch unverzahnt als Matrix mit r Zeilen angegeben werden.
%
%   s  :   erwartete Symbole fr 0 und 1 z.B. [-1 +1]
%
%  diag:   falls angegeben mit beliebigem Wert~=0: Trellisgraphik 
%          und Kontrollausgabe ins Kommandofenster.
%          Dies ist nur sinnvoll fr Constraint Length < 5 !
%          Diese Ausgaben gibt es auch, falls viterbi ganz ohne
%          Parameter aufgerufen wird.
%
%   d  :   dekodierte Bits
%
% Hinweis: Falls mit dem Trellis-Diagramm gearbeitet wird, erfolgt 
%          die Berechnung und Ausgabe der ersten zwei Schritte durch  
%          Fortschaltung mit beliebiger Tatatureingabe. Nach der 
%          vollstndigen Dekodierung wird der verbliebene Weg im 
%          Trellisdiagramm rot eingezeichnet. Dann kann das Diagramm
%          mit den Tasten -+ seitlich verschoben werden.
%
%          Smtliche Trelliswege werden nur bis zu einer Lnge von 
%          144 Bit verwaltet. ber diese Lnge hinaus darf nur noch 
%          ein endgltig entschiedener Weg existieren. Andernfalls 
%          wird eine Warnung ausgegeben, die immer ein Zeichen fr 
%          ein zu stark gestrtes Signal ist.
%

% Autor:  Klaus von der Heide,  e-mail:  v.d.heide@on-line.de

% Defaults
% ========
if nargin<1
   g = faltpoly(2,3); 
   diag = 1;
elseif nargin<4   
   diag = 0;
end
if nargin<2 
   info = [0 1 0 0 0 1 1 0 1 0];
   x = 2*faltcode(info,g,0,2)-1;
   x = x + 1.0*randn(size(x));
end
if nargin<3      s = [-1 +1]; end
if diag
   Generatormatrix  = binmatout(g)
   if nargin<1  Informationsbits = binmatout(info), end
   Kodewort = binmatout(x)
   if length(g)>4 error('Trellis-Diagramm fr Constraint-Length > 4 nicht sinnvoll !'); end
   if length(x)>16 error('Trellis-Diagramm fr Kodelngen > 16 nicht sinnvoll !'); end
end

% Konstanten
% ==========
[cr,cl] = size(g);               % inverse Koderate cr und Constraint Length cl
[xnz xns] = size(x);
if xnz==1  
   if rem(xns,cr)  error('Lnge von x passt nicht zu g'); end
   x = reshape(x,cr,xns/cr);     % x zu Matrix mit cr Zeilen machen
elseif xnz~=cr  
   error('Anzahl der Zeilen von x ungleich der von g');   
end
x = [x ones(cr,1)*s(1)];         % eine Null-Spalte an Tail anhngen
[xnz xns] = size(x);
fsz = 13;                        % FontSize fr Darstellung
zhn = 2^cl;                      % Anzahl der Zustnde
zhnh= zhn/2;                     % davon die Hlfte
for k=1:zhn                      % das letzte Bit der Kodierung aller Zustandsmuster
   y = 2*faltcode(reverse(num2binvec(k-1,cl)),g)-1;
   for r=1:cr  z(r,k) = y(r,cl); end;
end
k0 = 1:zhnh;
k1 = k0 + zhnh;
p1 = 2*k0;
p0 = p1 - 1;

% Trellisanfang
% =============
for k=1:zhn                      % alle Zustandsmuster als Anfangswege (reverse)
   uu    = reverse(num2binvec(k-1,cl));
   bw(k) = binvec2num(uu);
   xx    = 2*faltcode(uu,g)-1;
   c(k)  = sum(sum((xx(:,1:cl) - x(:,1:cl)).^2)); 
end
zw1 = zeros(1,zhn);              % komprimierte Wegereprsentation
zw2 = zeros(1,zhn);              % komprimierte Wegereprsentation
zw3 = zeros(1,zhn);              % komprimierte Wegereprsentation
zlr = cl;                        % Zhler fr Wegekompression
zl2 = 0;                         % Zhler fr Wegekompression
d   = [];
rvk1= ones(1,zhn);               % Rckverkettung
rvk2= ones(1,zhn);               % Rckverkettung
rvk3= ones(1,zhn);               % Rckverkettung
% Kontrollausgabe und Trellis-Graphik
if diag 
   fh = trellisdiag(cl,2);
   for k=1:zhn
      addtrellisdiag(fh,cl,2,1,0,num2binvec(bw(k),cl),0.8*[1 1 1]);
   end      
end

% Trellisschritt
% ==============
for t=cl+1:xns              % Zeitschleife ber Kodelnge
   c0 = c(p0);              % Kosten des ersten Vorgngerzustands
   c1 = c(p1);              % Kosten des zweiten Vorgngerzustands
%   xb = (ones(zhnh,1)*x(:,t)')';
   xb = x(:,t)*ones(1,zhnh);
   s0 = sum((xb - z(:,k0)).^2);  % Kosten einer 0 an t-ter Stelle
   s1 = sum((xb - z(:,k1)).^2);  % Kosten einer 1 an t-ter Stelle
   b0 = c0<c1;              % Vergleich der Kosten
   b1 = 1 - b0;
   pb = p0.*b0 + p1.*b1;
   cb = c0.*b0 + c1.*b1;
   c(k0)   = cb + s0;       % neue Kosten
   bwn(k0) = 2*bw(pb);      % neue Wegekompression
   rvkn(k0)= rvk1(pb);      % Kompressions-Rckverkettung
   c(k1)   = cb + s1;       % neue Kosten
   bwn(k1) = bwn(k0) + 1;   % neue Wegekompression
   rvkn(k1)= rvk1(pb);      % Kompressions-Rckverkettung
   bw      = bwn;           % Trellisschritt: Wegekompression
   rvk1    = rvkn;          % Trellisschritt: Kompressions-Rckverkettung
   % Wegekompression
   zlr = zlr + 1;
   if rem(zlr,16)==0 
      if zlr==48
         rvk = rvk3(rvk2(rvk1(1)));
         if any(rvk~=rvk3(rvk2(rvk1(2:zhn)))) 
            warning('Weg nach 144 Schritten noch nicht entschieden.')
         end
         d = [d  num2binvec(zw3(rvk),48)];
         zw3 = zw2;
         zw2 = zw1;
         zw1 = bw';
         rvk3= rvk2;
         rvk2= rvk1;
         rvk1= 1:zhn;
         zlr = 0;
      end    
      bw  = rem(bw,2^32);        
   end
   % Graphik
   if diag                 % Trellisdiagramm
      mnc = min(c);
      nnc = max(c) - mnc + 0.01;
      if t<=length(x)+1
         for k=1:zhn
            addtrellisdiag(fh,cl,2,1,0,num2binvec(bw(k),t),...
                           (c(k)-mnc)/nnc*[1 1 1]);
         end
      end
      if t<cl+4  pause; end   
   end
end

% Ausgabe
% =======
d = [d  num2binvec(zw3(rvk3(rvk2(rvk1(1)))),48)   ...
        num2binvec(zw2(rvk2(rvk1(1))),48)  ...
        num2binvec(zw1(rvk1(1)),48)  ...
        num2binvec(rem(bw(1),2^zlr),zlr)];
d = d(145:end-cl);
if diag | (nargin<1)       % Trellisdiagramm
   addtrellisdiag(fh,cl,2,1,0,d,'r');
end